package de.towerdefence.server.server.channels.matchmaking;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.towerdefence.server.match.confirmation.MatchConfirmationService;
import de.towerdefence.server.match.queue.MatchQueueService;
import de.towerdefence.server.player.Player;
import de.towerdefence.server.server.JsonWebsocketHandler;
import de.towerdefence.server.server.channels.matchmaking.bi.MatchSetSearchStateMessage;
import de.towerdefence.server.server.channels.matchmaking.in.MatchAcceptedMessage;
import de.towerdefence.server.server.channels.matchmaking.out.MatchAbortedMessage;
import de.towerdefence.server.server.channels.matchmaking.out.MatchEstablishedMessage;
import de.towerdefence.server.server.channels.matchmaking.out.MatchFoundMessage;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class MatchmakingWebsocketHandler extends JsonWebsocketHandler {
    protected final Map<Player, WebSocketSession> playerSessions = new ConcurrentHashMap<>();
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final MatchQueueService matchQueueService;
    private final MatchConfirmationService matchConfirmationService;


    public MatchmakingWebsocketHandler(
            SessionsService sessionsService,
            MatchQueueService matchQueueService,
            MatchConfirmationService matchConfirmationService
    ) {
        super(Channel.MATCHMAKING, sessionsService);
        this.matchQueueService = matchQueueService;
        this.matchConfirmationService = matchConfirmationService;
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        super.afterConnectionEstablished(session);
        playerSessions.put(sessionPlayers.get(session), session);
    }

    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) {
        try {
            String payload = message.getPayload();
            switch (objectMapper.readTree(payload).get("$id").asText()) {
                case MatchSetSearchStateMessage.MESSAGE_ID -> handleMatchSetSearchStateMessage(session, payload);
                case MatchAcceptedMessage.MESSAGE_ID -> handleMatchAcceptedMessage(session, payload);
                default -> this.closeSession(session, CloseStatus.BAD_DATA);
            }
        } catch (Exception ignored) {
            this.closeSession(session, CloseStatus.BAD_DATA);
        }
    }

    private void handleMatchSetSearchStateMessage(WebSocketSession session, String payload) throws Exception {
        MatchSetSearchStateMessage msg = objectMapper.readValue(payload, MatchSetSearchStateMessage.class);
        Player player = sessionPlayers.get(session);
        if (!msg.isSearching()) {
            this.matchQueueService.unQueuePlayer(player);
            return;
        }
        this.matchQueueService.queuePlayer(
                player,
                this::onFound,
                this::onQueued,
                this::onAbort,
                this::onEstablished
        );
    }

    private void onFound(Player player, String matchId, long created, long ttl) throws IOException {
        WebSocketSession session = playerSessions.get(player);
        MatchFoundMessage msg = new MatchFoundMessage(matchId, created, ttl);
        msg.send(session);
    }

    private void onQueued(Player player) throws IOException{
        WebSocketSession session = playerSessions.get(player);
        MatchSetSearchStateMessage msg = new MatchSetSearchStateMessage(true);
        msg.send(session);
    }

    private void onAbort(Player player, String matchId) {
        WebSocketSession session = playerSessions.get(player);
        MatchAbortedMessage msg = new MatchAbortedMessage(matchId);
        try {
            msg.send(session);
        } catch (IOException ignored) {
        }
    }

    private void onEstablished(Player player, String matchId, Player opponent) throws IOException {
        WebSocketSession session = playerSessions.get(player);
        MatchEstablishedMessage msg = new MatchEstablishedMessage(matchId, opponent.getUsername());
        msg.send(session);
    }

    private void handleMatchAcceptedMessage(WebSocketSession session, String payload) throws Exception {
        MatchAcceptedMessage msg = objectMapper.readValue(payload, MatchAcceptedMessage.class);
        Player player = sessionPlayers.get(session);
        if (msg.isAccepted()) {
            this.matchConfirmationService.accept(player, msg.getMatchId());
        } else {
            this.matchConfirmationService.decline(player, msg.getMatchId());
        }
    }
}
